<?php
namespace app\video\controller;

use app\video\controller\Common;

class Category extends Common
{
    private $pages = ['head','hot','news','views','types','tags'];

    public function _initialize()
    {
        parent::_initialize();
    }
    
    public function index()
    {
        if( isset($this->query['id']) ){
            $term = videoCategoryId($this->query['id']);
        }elseif( isset($this->query['slug']) ){
            $term = videoCategorySlug($this->query['slug']);
        }elseif( isset($this->query['name']) ){
            $term = videoCategoryName($this->query['name']);
        }else{
            $this->error(lang('video/error/params'),'video/index/index');
        }
        //数据为空
        if(!$term){
            $this->error(lang('video/error/empty'),'video/index/index');
        }
        //分页路径
        $term['pagePath']   = videoUrlCategory($term,'[PAGE]');
        //地址栏
        $term['pageSize']   = videoPageSize($term['term_limit'],'category');
        $term['pageNumber'] = $this->site['page'];
        $term['sortName']   = 'info_order desc,info_update_time';
        $term['sortOrder']  = 'desc';
        //SEO标签
        $term['seoTitle']       = videoSeo(DcEmpty($term['term_title'],$term['term_name']),$this->site['page']);
        $term['seoKeywords']    = videoSeo(DcEmpty($term['term_keywords'],$term['term_name']),$this->site['page']);
        $term['seoDescription'] = videoSeo(DcEmpty($term['term_description'],$term['term_name']),$this->site['page']);
        //分类列表
        if($term['term_action'] == 'index'){
            $this->item($term);
        }
        //队列列表
        if($term['term_slug'] == 'types'){
            $this->types($term);//所有分类
        }elseif($term['term_slug'] == 'tags'){
            $this->tags($term);//所有标签
        }
        //变量赋值
        $this->assign($term);
        //加载模板
        if($term['term_tpl']){
            return $this->fetch($term['term_tpl']);
        }
        //内置页面
        if( in_array($term['term_slug'], $this->pages) ){
            return $this->fetch($term['term_slug']);
        }
        //默认加载
        return $this->fetch('index');
    }
    
    //公共操作
    public function common()
    {
        $this->redirect('video/category/index', $this->query, 302);
    }
    
    //列表数据
    private function item($term=[])
    {
        $item = videoSelect([
            'cache'   => true,
            'status'  =>'normal',
            'term_id' => $term['term_id'],
            'limit'   => $term['pageSize'],
            'page'    => $term['pageNumber'],
            'sort'    => $term['sortName'],
            'order'   => $term['sortOrder'],
        ]);
        //变量赋值
        if($item){
            $this->assign($item);
        }
    }
    
    //所有分类
    private function types($term=[])
    {
        $item = videoCategorySelect([
            'cache'   => true,
            'status'  => 'normal',
            'action'  => 'index',
            'sort'    => 'term_count desc,term_id',
            'order'   => 'desc',
        ]);
        if($item){
            $this->assign('data', $item);
        }
    }
    
    //所有标签
    private function tags($term=[])
    {
        $item = videoTagSelect([
            'cache'   => true,
            'status'  => 'normal',
            'limit'   => $term['pageSize'],
            'page'    => $term['pageNumber'],
            'sort'    => 'term_count desc,term_id',
            'order'   => 'desc',
        ]);
        if($item){
            $this->assign($item);
        }
    }
}